package com.sun.xml.internal.ws.server.provider;

import com.sun.xml.internal.ws.fault.SOAPFaultBuilder;
import com.sun.xml.internal.ws.api.SOAPVersion;
import javax.xml.ws.soap.SOAPBinding;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.api.WSBinding;
import com.sun.xml.internal.ws.api.model.wsdl.WSDLPort;
import com.sun.xml.internal.ws.api.message.Packet;
import com.sun.xml.internal.ws.api.message.Message;

public abstract class ProviderArgumentsBuilder<T>
{
    public ProviderArgumentsBuilder() {
    }
    
    protected abstract Message getResponseMessage(Exception p0);
    
    protected Packet getResponse(Packet request, Exception e, WSDLPort port, WSBinding binding) {
        Message message = this.getResponseMessage(e);
        Packet response = request.createServerResponse(message, port, null, binding);
        return response;
    }
    
    public abstract T getParameter(Packet p0);
    
    protected abstract Message getResponseMessage(T p0);
    
    protected Packet getResponse(Packet request, @Nullable T returnValue, WSDLPort port, WSBinding binding) {
        Message message = null;
        if (returnValue != null) {
            message = this.getResponseMessage(returnValue);
        }
        Packet response = request.createServerResponse(message, port, null, binding);
        return response;
    }
    
    public static ProviderArgumentsBuilder<?> create(ProviderEndpointModel model, WSBinding binding) {
        if (model.datatype == Packet.class) {
            return new ProviderArgumentsBuilder.PacketProviderArgumentsBuilder(binding.getSOAPVersion());
        }
        return (binding instanceof SOAPBinding) ? SOAPProviderArgumentBuilder.create(model, binding.getSOAPVersion()) : XMLProviderArgumentBuilder.createBuilder(model, binding);
    }
    
    private static class PacketProviderArgumentsBuilder extends ProviderArgumentsBuilder<Packet>
    {
        private final SOAPVersion soapVersion;
        
        public PacketProviderArgumentsBuilder(SOAPVersion soapVersion) {
            this.soapVersion = soapVersion;
        }
        
        @Override
        protected Message getResponseMessage(Exception e) {
            return SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, null, e);
        }
        
        @Override
        public Packet getParameter(Packet packet) {
            return packet;
        }
        
        @Override
        protected Message getResponseMessage(Packet returnValue) {
            throw new IllegalStateException();
        }
        
        @Override
        protected Packet getResponse(Packet request, @Nullable Packet returnValue, WSDLPort port, WSBinding binding) {
            return returnValue;
        }
    }
}
